import pywaterml.aux
import pywaterml.analyzeData
import pywaterml.waterML

# __version__ = '0.0.14.dev1'
# __version__ = '1.1.1'
# __author__ = 'Elkin Giovanni Romero Bustamante'

from ._version import get_versions
__author__ = 'Elkin Giovanni Romero Bustamante'
__version__ = get_versions()['version']
del get_versions
